import { ILifetime } from '../Lifetime';
import { IServiceProvider } from '../ServiceProvider';
import { MockSetup } from './mockUtils';
import { DependencyOptions, Key, LifetimeConstructor, Selector, Stateful, StatefulDependencyOptions } from './types';
export declare class ServiceCollection<E = {}> {
    private readonly self;
    private readonly lifetimes;
    constructor(other?: ServiceCollection, lifetime?: ILifetime<any, E>);
    addMultiple<T>(func: (services: ServiceCollection<E>) => ServiceCollection<T>): ServiceCollection<T>;
    addStateful<T, P, KE>(name: keyof KE & Key<KE>, Dependency: StatefulDependencyOptions<T, P, E & {
        [key in keyof KE]: Stateful<P, T>;
    }>): ServiceCollection<{ [key in keyof KE | keyof E]: key extends keyof E ? E[key] : key extends keyof KE ? Stateful<P, T> : never; }>;
    addSingleton<T, KE>(name: keyof KE & Key<KE> & (keyof KE extends keyof E ? never : any), Dependency: DependencyOptions<T, E>): ServiceCollection<{ [key in keyof E | keyof KE]: key extends keyof E ? E[key] : key extends keyof KE ? T : never; }>;
    addScoped<T, KE>(name: keyof KE & Key<KE> & (keyof KE extends keyof E ? never : any), Dependency: DependencyOptions<T, E>): ServiceCollection<{ [key in keyof E | keyof KE]: key extends keyof E ? E[key] : key extends keyof KE ? T : never; }>;
    addTransient<T, KE>(name: keyof KE & Key<KE> & (keyof KE extends keyof E ? never : any), Dependency: DependencyOptions<T, E>): ServiceCollection<{ [key in keyof E | keyof KE]: key extends keyof E ? E[key] : key extends keyof KE ? T : never; }>;
    add<T, KE>(Lifetime: LifetimeConstructor, name: keyof KE & Key<KE> & (keyof KE extends keyof E ? never : any), Dependency: DependencyOptions<T, E>): ServiceCollection<{ [key in keyof E | keyof KE]: key extends keyof E ? E[key] : key extends keyof KE ? T : never; }>;
    get<T>(selector: Selector<T, E>): ILifetime<T, E> | undefined;
    remove<T, KE>(selector: Selector<T, E, KE>): ServiceCollection<{ [key in keyof Omit<E, keyof KE>]: E[key]; }>;
    build(): IServiceProvider<E>;
    buildMock(mock?: MockSetup<E>): IServiceProvider<E>;
    private extractFactory;
    private cloneLifetimes;
}
